@@ -7,12 +7,19 @@
 #include "../kwalletmanager_version.h"
 #include "konfigurator.h"
 
+#include <kauth_version.h>
 #include <KPluginFactory>
 #include <QInputDialog>
 #include <KWallet>
+#if KAUTH_VERSION >= QT_VERSION_CHECK(5, 92, 0)
+#include <KAuth/Action>
+#include <KAuth/ActionReply>
+#include <KAuth/ExecuteJob>
+#else
 #include <KAuthAction>
 #include <KAuthActionReply>
 #include <KAuthExecuteJob>
+#endif
 #include <KConfigGroup>
 #include <KMessageBox>
 
@@ -80,9 +87,7 @@ KWalletConfig::KWalletConfig(QWidget *parent, const QVariantList &args)
 
 }
 
-KWalletConfig::~KWalletConfig()
-{
-}
+KWalletConfig::~KWalletConfig() = default;
 
 void KWalletConfig::updateWalletLists()
 {
@@ -117,12 +122,12 @@ QString KWalletConfig::newWallet()
                                             &ok);
 
     if (!ok) {
-        return QString();
+        return {};
     }
 
     KWallet::Wallet *w = KWallet::Wallet::openWallet(n, topLevelWidget()->winId());
     if (!w) {
-        return QString();
+        return {};
     }
 
     delete w;
