@@ -13,39 +13,50 @@
 #include "registercreateactionmethod.h"
 #include "kwalletmanager_debug.h"
 
-#include <kxmlgui_version.h>
+#include <kwindowsystem_version.h>
+#include <KWindowSystem>
 #include <KLocalizedString>
-#include <QAction>
 #include <KConfig>
 #include <KMessageBox>
 #include <KStandardAction>
+#include <knotifications_version.h>
 #include <KStatusNotifierItem>
 #include <KWallet>
 #include <KXMLGUIFactory>
-#include <QIcon>
 #include <KActionCollection>
 #include <KConfigGroup>
 #include <KTar>
 #include <KIO/CommandLauncherJob>
 
+#include <QAction>
+#include <QCommandLineParser>
+#include <QIcon>
 #include <QRegExp>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QRegExpValidator>
+#endif
 #include <QTimer>
 #include <QFileDialog>
 #include <QDialog>
 #include <QLineEdit>
+#include <QFile>
+#include <QFileInfo>
+#include <QMimeDatabase>
+#include <QMimeType>
 
 
-KWalletManager::KWalletManager(QWidget *parent, const QString &name, Qt::WindowFlags f)
-    : KXmlGuiWindow(parent, f)
+KWalletManager::KWalletManager(QCommandLineParser *commandLineParser)
+    : KXmlGuiWindow()
+    , _commandLineParser(commandLineParser)
 {
     _kwalletdLaunch = false;
     _shuttingDown = false;
     m_kwalletdModule = nullptr;
-    setObjectName(name);
     RegisterCreateActionsMethod::createActions(actionCollection());
 
     QTimer::singleShot(0, this, &KWalletManager::beginConfiguration);
+
+    processParsedCommandLine(ProgramStart);
 }
 
 void KWalletManager::beginConfiguration() {
@@ -192,7 +203,11 @@ bool KWalletManager::queryClose()
         if (!_tray) {
             qApp->quit();
         } else {
+#if KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+            _tray->hideAssociatedWidget();
+#else
             hide();
+#endif
         }
         return false;
     }
@@ -298,19 +313,23 @@ void KWalletManager::possiblyRescan(const QString &app, const QString &oldOwner,
 void KWalletManager::createWallet()
 {
     QString txt = i18n("Please choose a name for the new wallet:");
-    QRegExpValidator validator(QRegExp(QLatin1String(R"(^[\w\^\&\'\@\{\}\[\]\,\$\=\!\-\#\(\)\%\.\+\_\s]+$)")), this);
 
     if (!KWallet::Wallet::isEnabled()) {
         // FIXME: KMessageBox::warningYesNo(this, i1_8n("KWallet is not enabled.  Do you want to enable it?"), QString(), i18n("Enable"), i18n("Keep Disabled"));
         return;
     }
-
+    // TODO port to QRegularExpressionValidator
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    QRegExpValidator validator(QRegExp(QLatin1String(R"(^[\w\^\&\'\@\{\}\[\]\,\$\=\!\-\#\(\)\%\.\+\_\s]+$)")), this);
+#endif
     QDialog nameDialog(this);
     nameDialog.setWindowTitle(i18n("New Wallet"));
     nameDialog.setLayout(new QVBoxLayout);
     nameDialog.layout()->addWidget(new QLabel(txt));
     auto lineEdit = new QLineEdit;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     lineEdit->setValidator(&validator);
+#endif
     nameDialog.layout()->addWidget(lineEdit);
 
     auto buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
@@ -474,3 +493,96 @@ bool KWalletManager::canIgnoreUnsavedChanges()
     int rc = KMessageBox::warningYesNo(this, i18n("Ignore unsaved changes?"));
     return (rc == KMessageBox::Yes);
 }
+
+void KWalletManager::handleActivate(const QStringList &arguments, const QString &workingDirectory)
+{
+    Q_UNUSED(workingDirectory);
+
+    // DBus activation
+    if (arguments.isEmpty()) {
+        activateForStartLikeCall(true);
+        return;
+    }
+
+    // KDBusService::Unique handling
+    _commandLineParser->parse(arguments);
+
+    processParsedCommandLine(RemoteCall);
+}
+
+void KWalletManager::processParsedCommandLine(CommandLineOrigin commandLineOrigin)
+{
+    const bool shoWWindow = _commandLineParser->isSet(QStringLiteral("show"));
+    if (commandLineOrigin == ProgramStart) {
+        if (shoWWindow)
+            show();
+    } else {
+        activateForStartLikeCall(shoWWindow);
+    }
+
+    if (_commandLineParser->isSet(QStringLiteral("kwalletd"))) {
+        kwalletdLaunch();
+    }
+
+    const QStringList positionalArguments = _commandLineParser->positionalArguments();
+    QStringList localFiles;
+    for (const QString &arg : positionalArguments) {
+        const QString fn = QFileInfo(arg).absoluteFilePath();
+        if (QFile::exists(fn)) {
+            localFiles.append(fn);
+        } else {
+            openWallet(arg);
+        }
+    }
+    tryOpenWalletFiles(localFiles);
+}
+
+void KWalletManager::handleOpen(const QList<QUrl> &urls)
+{
+    QStringList localFiles;
+    for (const QUrl& url : urls) {
+        localFiles.append(url.toLocalFile());
+    }
+    tryOpenWalletFiles(localFiles);
+    activateForStartLikeCall(false);
+}
+
+void KWalletManager::tryOpenWalletFiles(const QStringList &localFiles)
+{
+    QMimeDatabase mimeDb;
+    for (const QString &localFile : localFiles) {
+        if (QFile::exists(localFile)) {
+            QMimeType mt = mimeDb.mimeTypeForFile(localFile, QMimeDatabase::MatchContent);
+
+            if (mt.isValid() && mt.inherits(QStringLiteral("application/x-kwallet"))) {
+                openWalletFile(localFile);
+            }
+        }
+    }
+}
+
+void KWalletManager::activateForStartLikeCall(bool showWindow)
+{
+    if (showWindow) {
+        show();
+    }
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+    KWindowSystem::updateStartupId(windowHandle());
+#else
+    if (KWindowSystem::isPlatformWayland()) {
+        KWindowSystem::setCurrentXdgActivationToken(qEnvironmentVariable("XDG_ACTIVATION_TOKEN"));
+        //
+    }
+#endif
+    if (showWindow) {
+        if (isMinimized())
+            KWindowSystem::unminimizeWindow(winId());
+        else if (_tray && !isVisible()) // TODO: how does this relate to show() above?
+            _tray->activate(QPoint());
+
+        if (!isActiveWindow()) {
+            raise();
+            KWindowSystem::activateWindow(winId());
+        }
+    }
+}
